/* Author: Derek O Reilly, Dundalk Institute of Technology, Ireland. */

class Shell extends GameObject
{
    /* Each gameObject MUST have a constructor() and a render() method.        */
    /* If the object animates, then it must also have an updateState() method. */

    constructor(explosionImage, shellExplosionSound, x, y, direction)
    {
        super(5); /* as this class extends from GameObject, you must always call super() */
        
        this.explosionImage = explosionImage;
        this.shellExplosionSound = shellExplosionSound;
        this.x = x;
        this.y = y;
        this.explosionTargetX = x;
        this.explosionTargetY = y;
        this.direction = direction;

        /* define the maximum range of the shell */
        /* the shell will explode here if it has not hit a target beforehand */
        this.shellRange = 200;
        this.distanceShellTravelled = gameObjects[TANK].getSize() / 2;  // the shell starts from the front of the tank's turret
    }

    updateState()
    {
        if (this.distanceShellTravelled < this.shellRange)
        {
            this.distanceShellTravelled ++;
            this.explosionTargetX = this.x + (this.distanceShellTravelled * Math.sin(Math.radians(this.direction)));
            this.explosionTargetY = this.y - (this.distanceShellTravelled * Math.cos(Math.radians(this.direction)));
        }
        else
        {
            this.stopAndHide();
            gameObjects[EXPLOSION] = new Explosion(this.explosionImage, this.shellExplosionSound, this.explosionTargetX, this.explosionTargetY, 120);
            gameObjects[EXPLOSION].start();
        }
    }

    getX()
    {
        return this.explosionTargetX;
    }

    getY()
    {
        return this.explosionTargetY;
    }

    getRange()
    {
        return this.shellRange;
    }

    isFiring()
    {
        return this.gameObjectIsDisplayed;
    }
}