/* Author: Derek O Reilly, Dundalk Institute of Technology, Ireland.       */
/* A CanvasGame that implements collision detection.                       */


class TankCanvasGame extends CanvasGame
{
    constructor()
    {
        super();

        this.numberOfEnemytanksDestroyed = 0;
    }

    collisionDetection()
    {
        /* Collision detection for the player tank bumping into an enemy tank */
        for (let i = 0; i < enemyTanks.length; i++)
        {
            if ((enemyTanks[i].pointIsInsideTank(gameObjects[TANK].getFrontLeftCornerX(), gameObjects[TANK].getFrontLeftCornerY())) ||
                    (enemyTanks[i].pointIsInsideTank(gameObjects[TANK].getFrontRightCornerX(), gameObjects[TANK].getFrontRightCornerY())))
            {
                enemyTanks[i].reverse(5);
                enemyTanks[i].setDirection(enemyTanks[i].getDirection() + 10); // turn away from the river, so that the tank does not get stuck in one place
            }
        }


        /* Collision detection for the enemy tanks bumping into each other */
        for (let i = 0; i < enemyTanks.length; i++)
        {
            /* check if enemy tank bumps into player tank */
            if ((gameObjects[TANK].pointIsInsideTank(enemyTanks[i].getFrontLeftCornerX(), enemyTanks[i].getFrontLeftCornerY())) ||
                    (gameObjects[TANK].pointIsInsideTank(enemyTanks[i].getFrontRightCornerX(), enemyTanks[i].getFrontRightCornerY())))
            {
                enemyTanks[i].reverse(5);
                enemyTanks[i].setDirection(enemyTanks[i].getDirection() + 10); // turn away from the river, so that the tank does not get stuck in one place
            }

            /* check if enemy tank bumps into another enemy tank */
            for (let j = 0; j < enemyTanks.length; j++)
            {
                if (i !== j)
                {
                    if ((enemyTanks[i].pointIsInsideTank(enemyTanks[j].getFrontLeftCornerX(), enemyTanks[j].getFrontLeftCornerY())) ||
                            (enemyTanks[i].pointIsInsideTank(enemyTanks[j].getFrontRightCornerX(), enemyTanks[j].getFrontRightCornerY())))
                    {
                        enemyTanks[i].reverse(3);
                        enemyTanks[i].setDirection(enemyTanks[i].getDirection() + 10); // turn away from the river, so that the tank does not get stuck in one place
                    }
                }
            }
        }


        /* Collision detection of the player tank with the river */
        if (gameObjects[TANK].collidedWithRiver())
        {
            gameObjects[TANK].reverse();
        }

        /* Collision detection for enemy tanks with the river */
        for (let i = 0; i < enemyTanks.length; i++)
        {
            if (enemyTanks[i].collidedWithRiver())
            {
                enemyTanks[i].reverse();
                enemyTanks[i].setDirection(enemyTanks[i].getDirection() + 10); // turn away from the river, so that the tank does not get stuck in one place
            }
        }

        /* Collision detection for a shell that is firing */
        if (gameObjects[SHELL] === undefined)
        {
            return;
        }
        for (let i = 0; i < enemyTanks.length; i++)
        {
            if (gameObjects[SHELL].isFiring())
            {
                if ((enemyTanks[i].isDisplayed()) && (enemyTanks[i].pointIsInsideTank(gameObjects[SHELL].getX(), gameObjects[SHELL].getY())))
                {
                    enemyTanks[i].stopAndHide();
                    gameObjects[EXPLOSION] = new Explosion(explosionImage, shellExplosionSound, enemyTanks[i].getX(), enemyTanks[i].getY(), 200);
                    gameObjects[EXPLOSION].start();
                    gameObjects[SHELL].stopAndHide();

                    this.numberOfEnemytanksDestroyed++;
                    if (this.numberOfEnemytanksDestroyed === numberOfEnemyTanks)
                    {
                        /* Player has won                                                                                             */
                        /* Have a two second delay to show the last enemy tank blowing up beofore displaying the 'Game Over!' message */
                        setInterval(function ()
                        {
                            for (let j = 0; j < gameObjects.length; j++) /* stop all gameObjects from animating */
                            {
                                gameObjects[j].stopAndHide();
                            }
                            gameObjects[TANK].stopMoving(); // turn off tank moving sound
                            gameObjects[BACKGROUND].start();
                            gameObjects[WIN_MESSAGE] = new StaticText("Game Over!", 5, 270, "Times Roman", 100, "red");
                            gameObjects[WIN_MESSAGE].start(); /* render win message */
                        }, 2000);
                    }
                }
            }
        }
    }

    render()
    {
        super.render();
        for (let i = 0; i < enemyTanks.length; i++)
        {
            if (enemyTanks[i].isDisplayed())
            {
                enemyTanks[i].render();
            }
        }
    }
}